@@ -519,6 +519,13 @@ source_file_property(File, P) :-
 
 property_source_file(modified(Time), File) :-
     '$time_source_file'(File, Time, user).
+property_source_file(source(Source), File) :-
+    (   '$source_file_property'(File, from_state, true)
+    ->  Source = state
+    ;   '$source_file_property'(File, resource, true)
+    ->  Source = resource
+    ;   Source = file
+    ).
 property_source_file(module(M), File) :-
     (   nonvar(M)
     ->  '$current_module'(M, File)
@@ -627,9 +634,12 @@ prolog_load_context(reloading, true) :-
 %
 %   Remove all traces of loading file.
 
+:- dynamic system:'$resolved_source_path'/2.
+
 unload_file(File) :-
     (   canonical_source_file(File, Path)
-    ->  '$unload_file'(Path)
+    ->  '$unload_file'(Path),
+        retractall(system:'$resolved_source_path'(_, Path))
     ;   true
     ).
 
@@ -823,6 +833,12 @@ property_predicate(implementation_module(IM), M:Head) :-
     ->  IM = LoadModule
     ;   M = IM
     ).
+property_predicate(iso, _:Head) :-
+    callable(Head),
+    !,
+    goal_name_arity(Head, Name, Arity),
+    current_predicate(system:Name/Arity),
+    '$predicate_property'(iso, system:Head).
 property_predicate(Property, Pred) :-
     define_or_generate(Pred),
     '$predicate_property'(Property, Pred).
@@ -1263,7 +1279,7 @@ format(Fmt) :-
                  *            FILES             *
                  *******************************/
 
-%       absolute_file_name(+Term, -AbsoluteFile)
+%!  absolute_file_name(+Term, -AbsoluteFile)
 
 absolute_file_name(Name, Abs) :-
     atomic(Name),
