@@ -4,6 +4,7 @@
     E-mail:        jeffrose@acm.org
     WWW:           http://www.swi-prolog.org
     Copyright (c)  2012-2013, Jeffrey Rosenwald
+		   2018, CWI Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -33,11 +34,15 @@
 */
 
 :- module(udp_broadcast,
-             [
-             udp_host_to_address/2,    % ? Host, ? Address
-             udp_broadcast_initialize/2,   % +IPAddress, +Subnet
-             udp_broadcast_service/2   % ? Domain, ? Address
-             ]).
+          [ udp_host_to_address/2,             % ?Host, ?Address
+            udp_broadcast_initialize/2,        % +IPAddress, +Options
+            udp_broadcast_service/2            % ?Scope, ?Address
+          ]).
+:- use_module(library(socket)).
+:- use_module(library(broadcast)).
+:- use_module(library(time)).
+:- use_module(library(option)).
+
 
 /** <module> A UDP Broadcast Bridge
 
@@ -50,8 +55,9 @@ any member on your local IP subnetwork that also has this library loaded
 may hear and respond to your broadcasts.
 
 This  module  has  only  two  public  predicates.  When  the  module  is
-initialized, it starts a two listener threads that listen for broadcasts
-from others, received as UDP datagrams.
+initialized using udp_broadcast_initialize/2, it starts   a two listener
+threads  that  listen  for  broadcasts  from  others,  received  as  UDP
+datagrams.
 
 Unlike TIPC broadcast, UDP broadcast has only one scope, =udp_subnet=. A
 broadcast/1 or broadcast_request/1 that is not  directed to the listener
@@ -200,8 +206,8 @@ and subtle differences that must be taken into consideration:
     (or none at all). No ordering of replies is implied.
 
     * Prolog terms are sent to others after first converting them to
-    atoms using term_to_atom/2. Passing real numbers this way may
-    result in a substantial truncation of precision.
+    atoms using term_string/3.  Serialization does not deal with cycles,
+    attributes or sharing.
 
     * The broadcast model is based on anonymity and a presumption of
     trust--a perfect recipe for compromise. UDP is an Internet protocol.
@@ -219,39 +225,29 @@ and subtle differences that must be taken into consideration:
     members of the local subnet. This security measure only keeps honest
     people honest!
 
-@author    Jeffrey Rosenwald (JeffRose@acm.org)
-@license   LGPL
+@author    Jeffrey Rosenwald (JeffRose@acm.org), Jan Wielemaker
+@license   BSD-2
 @see       tipc.pl
 */
 
-:- use_module(library(socket)).
-:- use_module(library(broadcast)).
-:- use_module(library(time)).
+:- multifile
+    udp_term_string_hook/2,                     % ?Term, ?String
+    udp:host_to_address/2.                      % +Host, -Address
 
-:- require([ thread_self/1
-           , forall/2
-           , term_to_atom/2
-           , thread_send_message/2
-           , catch/3
-           , setup_call_cleanup/3
-           , thread_create/3
-           ]).
-
-% %     ~>(:P, :Q) is nondet.
-% %     eventually_implies(P, Q) is nondet.
-%    asserts temporal Liveness (something good happens, eventually) and
-%    Safety (nothing bad ever happens) properties. Analogous to the
+%!   ~>(:P, :Q) is nondet.
+%
+%    asserts temporal Liveness (something good  happens, eventually) and
+%    Safety (nothing bad ever  happens)   properties.  Analogous  to the
 %    "leads-to" operator of Owicki and Lamport, 1982. Provides a sort of
 %    lazy implication described informally as:
 %
-%    * Liveness: For all possible outcomes, P -> Q, eventually.
-%    * Safety: For all possible outcomes, (\+P ; Q), is invariant.
+%      - Liveness: For all possible outcomes, P -> Q, eventually.
+%      - Safety: For all possible outcomes, (\+P ; Q), is invariant.
 %
-%  Described practically:
+%    Described practically:
 %
 %    P ~> Q, declares that if P is true, then Q must be true, now or at
 %    some point in the future.
-%
 
 :- meta_predicate ~>(0,0).
 :- op(950, xfy, ~>).
@@ -259,16 +255,18 @@ and subtle differences that must be taken into consideration:
 ~>(P, Q) :-
     setup_call_cleanup(P,
                        (true; fail),
-                       (   Q -> true;
-                       throw(error(goal_failed(Q), context(~>, _))))
-                      ).
+                       (   Q -> true
+                       ;   throw(error(goal_failed(Q), context(~>, _))))
+                       ).
 
 :- meta_predicate safely(0).
 
 safely(Predicate) :-
     catch(Predicate, Err,
-          (Err == '$aborted' -> (!, fail);
-          print_message(error, Err), fail)).
+          (   Err == '$aborted'
+          ->  !, fail
+          ;   print_message(error, Err), fail
+          )).
 
 :- meta_predicate make_thread(0, +).
 
@@ -282,7 +280,7 @@ join_thread(Id) :-
     thread_join(Id, exception('$aborted')).
 
 make_thread(Goal, Options) :-
-    thread_create(safely(Goal), Id, [ detached(false) | Options ])
+    thread_create(safely(Goal), Id, Options)
       ~> join_thread(Id).
 
 udp_broadcast_address(IPAddress, Subnet, BroadcastAddress) :-
@@ -295,19 +293,19 @@ udp_broadcast_address(IPAddress, Subnet, BroadcastAddress) :-
     B3 is A3 \/ (S3 xor 255),
     B4 is A4 \/ (S4 xor 255).
 
-%!  udp_broadcast_service(?Domain, ?Address) is nondet.
-%   provides the UDP broadcast address for a given Domain. At present,
-%   only one domain is supported, =|udp_subnet|=.
+%!  udp_broadcast_service(?Scope, ?Address) is nondet.
 %
+%   provides the UDP broadcast address for   a  given Scope. At present,
+%   only one scope is supported, =|udp_subnet|=.
 
 %  The following are defined at initialization:
 :- dynamic
     udp_subnet_member/1,      % +IpAddress:Port
-    udp_broadcast_service/2.  % ?Domain, ?BroadcastAddress:Port
+    udp_broadcast_service/2.  % ?Scope, ?BroadcastAddress:Port
 
 :- volatile
     udp_subnet_member/1,      % +IpAddress:Port
-    udp_broadcast_service/2.  % ?Domain, ?BroadcastAddress:Port
+    udp_broadcast_service/2.  % ?Scope, ?BroadcastAddress:Port
 %
 %  Here's a UDP bridge to Prolog's broadcast library
 %
@@ -354,20 +352,18 @@ black_list(tipc_zone(_,_)).
 %
 ld_dispatch(_S, '$udp_request'(Term), _From) :-
     black_list(Term), !, fail.
-
 ld_dispatch(_S, Term, _From) :-
     black_list(Term), !, fail.
-
 ld_dispatch(S, '$udp_request'(wru(Name)), From) :-
     !, gethostname(Name),
-    term_to_atom(wru(Name), Atom),
-    udp_send(S, Atom, From, []).
-
+    udp_term_string(wru(Name), Message),
+    udp_send(S, Message, From, []).
 ld_dispatch(S, '$udp_request'(Term), From) :-
-    !, forall(broadcast_request(Term),
-          (   term_to_atom(Term, Atom),
-              udp_send(S, Atom, From, []))).
-
+    !,
+    forall(broadcast_request(Term),
+           ( udp_term_string(Term, Message),
+             udp_send(S, Message, From, [])
+           )).
 ld_dispatch(_S, Term, _From) :-
     safely(broadcast(Term)).
 
@@ -403,147 +399,228 @@ udp_listener_daemon2(Parent) :-
     safely(dispatch_traffic(S, S1)).
 
 dispatch_traffic(S, S1) :-
-    udp_receive(S, Data, From,
-                [as(atom), max_message_size(65535)]),
-    udp_subnet_member(From),  % ignore all traffic that is foreign to my subnet
-    term_to_atom(Term, Data),
-    with_mutex(udp_broadcast, ld_dispatch(S1, Term, From)),
-    !,
+    udp_receive(S, Data, From, [max_message_size(65535)]),
+    (   udp_subnet_member(From),    % ignore traffic that is foreign to my subnet
+        udp_term_string(Term, Data) % only accept valid data
+    ->  with_mutex(udp_broadcast, ld_dispatch(S1, Term, From))
+    ;   true
+    ),
     dispatch_traffic(S, S1).
 
 start_udp_listener_daemon :-
-    catch(thread_property(udp_listener_daemon2, status(running)),_, fail),
-
+    catch(thread_property(udp_listener_daemon2, status(running)),
+          error(_,_),
+          fail),
     !.
-
 start_udp_listener_daemon :-
     thread_self(Self),
     thread_create(udp_listener_daemon2(Self), _,
-           [alias(udp_listener_daemon2), detached(true)]),
+                  [ alias(udp_listener_daemon2),
+                    detached(true)
+                  ]),
     call_with_time_limit(6.0,
                          thread_get_message(udp_listener_daemon_ready)).
 
-:- multifile udp:host_to_address/2.
-%
 broadcast_listener(udp_host_to_address(Host, Addr)) :-
     udp:host_to_address(Host, Addr).
-
-broadcast_listener(udp_broadcast_service(Class, Addr)) :-
-    udp_broadcast_service(Class, Addr).
-
+broadcast_listener(udp_broadcast_service(Scope, Addr)) :-
+    udp_broadcast_service(Scope, Addr).
 broadcast_listener(udp_subnet(X)) :-
     udp_broadcast(X, udp_subnet, 0.250).
-
 broadcast_listener(udp_subnet(X, Timeout)) :-
     udp_broadcast(X, udp_subnet, Timeout).
 
+%!  udp_broadcast(+What, +Scope, +TimeOut)
 %
+%   Send a broadcast request to my UDP peers in Scope. What is either of
+%   the shape `Term:Address` to send Term to a specific address or query
+%   the address from which term is answered or it is a plain `Term`.
 %
+%   If `Term` is  nonground,  it  is   considered  is  a  _request_ (see
+%   broadcast_request/1) and the predicate  succeeds   for  each  answer
+%   received within TimeOut seconds. If Term is ground it is considered
+%   an asynchronous broadcast and udp_broadcast/3 is deterministic.
+
+udp_broadcast(Term:To, _Scope, _Timeout) :-
+    ground(Term), ground(To),           % broadcast to single listener
+    !,
+    udp_basic_broadcast(_S, _Port, Term, To),
+    !.
+udp_broadcast(Term, Scope, _Timeout) :-
+    ground(Term),                       % broadcast to all listeners
+    !,
+    udp_broadcast_service(Scope, Address),
+    udp_basic_broadcast(_S, _Port, Term, Address),
+    !.
+udp_broadcast(Term:Address, _Scope, Timeout) :-
+    ground(Address),                    % request to single listener
+    !,
+    udp_basic_broadcast(S, Port, '$udp_request'(Term), Address),
+    udp_br_collect_replies(S, Port, Timeout, Term:Address).
+udp_broadcast(Term:From, Scope, Timeout) :-
+    !,                                  % request to all listeners, collect sender
+    udp_broadcast_service(Scope, Address),
+    udp_basic_broadcast(S, Port, '$udp_request'(Term), Address),
+    udp_br_collect_replies(S, Port, Timeout, Term:From).
+udp_broadcast(Term, Scope, Timeout) :-  % request to all listeners
+    udp_broadcast(Term:_, Scope, Timeout).
+
+
+%!  udp_basic_broadcast(-S, ?Port, +Term, +Address)
+%
+%   Create a UDP private socket and use it   to send Term to Address. If
+%   Address is our broadcast address, set the socket in broadcast mode.
+%
+%   This predicate succeeds with a choice   point. Committing the choice
+%   point closes S.
+
 udp_basic_broadcast(S, Port, Term, Address) :-
     udp_socket(S)
       ~> tcp_close_socket(S),
 
     (   udp_broadcast_service(udp_subnet, Address)
-           -> tcp_setopt(S, broadcast)
-           ; true
+    ->  tcp_setopt(S, broadcast)
+    ;   true
     ),
 
     tcp_bind(S, Port),  % find our own ephemeral Port
-    term_to_atom(Term, Atom),
+    udp_term_string(Term, String),
 
     (   udp_subnet_member(Address)  % talk only to your local subnet
-        -> safely(udp_send(S, Atom, Address, []))
-        ;  true).
+    ->  safely(udp_send(S, String, Address, []))
+    ;   true
+    ).
 
-% directed broadcast to a single listener
-udp_broadcast(Term:To, _Scope, _Timeout) :-
-    ground(Term), ground(To),
-    !,
-    udp_basic_broadcast(_S, _Port, Term, To),
-
-    !.
-
-% broadcast to all listeners
-udp_broadcast(Term, Scope, _Timeout) :-
-    ground(Term),
-    !,
-    udp_broadcast_service(Scope, Address),
-    udp_basic_broadcast(_S, _Port, Term, Address),
-
-    !.
-
-% directed broadcast_request to a single listener
-udp_broadcast(Term:Address, _Scope, Timeout) :-
-    ground(Address),
-    !,
-    udp_basic_broadcast(S, Port, '$udp_request'(Term), Address),
-    udp_br_collect_replies(S, Port, Timeout, Term:Address).
-
-% broadcast_request to all listeners returning responder port-id
-udp_broadcast(Term:From, Scope, Timeout) :-
-    !, udp_broadcast_service(Scope, Address),
-    udp_basic_broadcast(S, Port, '$udp_request'(Term), Address),
-    udp_br_collect_replies(S, Port, Timeout, Term:From).
-
-% broadcast_request to all listeners ignoring responder port-id
-udp_broadcast(Term, Scope, Timeout) :-
-    udp_broadcast(Term:_, Scope, Timeout).
-
-udp_br_send_timeout(Port) :-
-    udp_socket(S)
-      ~> tcp_close_socket(S),
-    udp_send(S, '$udp_br_timeout', localhost:Port, []),
-
-    !.
-
-udp_br_collect_replies(S, Port, Timeout, Term:From) :-
-    alarm(Timeout, udp_br_send_timeout(Port), Id, [remove(false)])
-      ~> remove_alarm(Id),
-
-    tcp_setopt(S, dispatch(false)),
+%!  udp_br_collect_replies(+Socket, +Port, +TimeOut, -TermAndFrom) is nondet.
+%
+%   Collect replies on Socket for  TimeOut   seconds.  Succeed  for each
+%   received message.
 
+udp_br_collect_replies(S, _Port, Timeout, Term:From) :-
+    tcp_getopt(S, file_no(Fd)),
+    get_time(Start),
+    Deadline is Start+Timeout,
     repeat,
-    udp_receive(S, Atom, From1, [as(atom)]),
-    (   (Atom \== '$udp_br_timeout')
-        -> (From1 = From, safely(term_to_atom(Term, Atom)))
-        ;  (!, fail)).
+       get_time(Now),
+       (   SingleTMO is Deadline - Now,
+           SingleTMO > 0,
+           wait_for_input([Fd], [Fd], SingleTMO)
+       ->  udp_receive(S, String, From, [max_message_size(65535)]),
+           safely(udp_term_string(Term, String))
+       ;   !,
+           fail
+       ).
 
 %!  udp_host_to_address(?Service, ?Address) is nondet.
 %
-%   locates a UDP service by name. Service  is an atom or grounded term
-%   representing the common name  of  the   service.  Address  is a UDP
+%   locates a UDP service by name. Service   is an atom or grounded term
+%   representing the common name  of  the   service.  Address  is  a UDP
 %   address structure. A server may advertise   its  services by name by
-%   including  the  fact,    udp:host_to_address(+Service,   +Address),
+%   including   the   fact,   udp:host_to_address(+Service,   +Address),
 %   somewhere in its source. This predicate can  also be used to perform
 %   reverse searches. That is it  will  also   resolve  an  Address to a
 %   Service name.
-%
 
 udp_host_to_address(Host, Address) :-
     broadcast_request(udp_subnet(udp_host_to_address(Host, Address))).
 
-%!  udp_initialize is semidet.
-%   See udp:udp_initialize/0
+%!  udp_broadcast_initialize(+IPAddress, +Options) is semidet.
 %
-%:- multifile udp:udp_stack_initialize/0.
-
-
-%!  udp_broadcast_initialize(+IPAddress, +SubnetMask) is semidet.
-%   causes any required runtime initialization to occur. At present,
-%   proper operation of UDP broadcast depends on local information
-%   that is not easily obtained mechanically. In order to determine
-%   the appropriate UDP broadcast address, you must supply the
-%   IPAddress and SubnetMask for the node that is running this module.
-%   These data are supplied in the form of ip/4 terms. This is now
-%   required to be included in an applications intialization directive.
+%   Initialized UDP broadcast bridge. IPAddress is the IP address on the
+%   network we want to broadcast on.  IP addresses are terms ip(A,B,C,D)
+%   or an atom or string of the format =|A.B.C.D|=.   Options processed:
 %
+%     - subnet_mask(+SubNet)
+%     Subnet to broadcast on.  This uses the same syntax as IPAddress.
+%     Default classifies the network as class A, B or C depending on
+%     the the first octet and applies the default mask.
+%     - port(+Port)
+%     Public port to use.  Default is 20005.
+%
+%   For compatibility reasons Options may be the subnet mask.
+
+udp_broadcast_initialize(IP, Options) :-
+    nonvar(Options),
+    Options = ip(_,_,_,_),
+    !,
+    udp_broadcast_initialize(IP, [subnet_mask(Options)]).
+udp_broadcast_initialize(IP, Options) :-
+    to_ip4(IP, IPAddress),
+    option(subnet_mask(Subnet), Options, _),
+    mk_subnet(Subnet, IPAddress, Subnet4),
+    option(port(Port), Options, 20005),
 
-udp_broadcast_initialize(IPAddress, Subnet) :-
     retractall(udp_broadcast_service(_,_)),
     retractall(udp_subnet_member(_)),
 
-    udp_broadcast_address(IPAddress, Subnet, BroadcastAddr),
-    assert(udp_broadcast_service(udp_subnet, BroadcastAddr:20005)),
-    assert(udp_subnet_member(Address:_Port) :- udp_broadcast_address(Address, Subnet, BroadcastAddr)),
+    udp_broadcast_address(IPAddress, Subnet4, BroadcastAddr),
+    assert(udp_broadcast_service(udp_subnet, BroadcastAddr:Port)),
+    assert((udp_subnet_member(Address:_Port) :-
+               udp_broadcast_address(Address, Subnet4, BroadcastAddr))),
 
     start_udp_listener_daemon.
+
+to_ip4(Atomic, ip(A,B,C,D)) :-
+    atomic(Atomic),
+    !,
+    (   split_string(Atomic, ".", "", Strings),
+        maplist(number_string, [A,B,C,D], Strings)
+    ->  true
+    ;   syntax_error(illegal_ip_address)
+    ).
+to_ip4(IP, IP).
+
+mk_subnet(Var, IP, Subnet) :-
+    var(Var),
+    !,
+    (   default_subnet(IP, Subnet)
+    ->  true
+    ;   domain_error(ip_with_subnet, IP)
+    ).
+mk_subnet(Subnet, _, Subnet4) :-
+    to_ip4(Subnet, Subnet4).
+
+default_subnet(ip(A,_,_,_), ip(A,0,0,0)) :-
+    between(1,126, A), !.
+default_subnet(ip(A,B,_,_), ip(A,B,0,0)) :-
+    between(128,191, A), !.
+default_subnet(ip(A,B,C,_), ip(A,B,C,0)) :-
+    between(192,223, A), !.
+
+
+		 /*******************************
+		 *             HOOKS		*
+		 *******************************/
+
+%!  udp_term_string(+Term, -String) is det.
+%!  udp_term_string(-Term, +String) is semidet.
+%
+%   Serialize an arbitrary Prolog  term  as   a  string.  The  string is
+%   prefixed by a magic key to ensure   we only accept messages that are
+%   meant for us.
+%
+%   In mode (+,-), Term is written with the options ignore_ops(true) and
+%   quoted(true).
+%
+%   This predicate first calls  udp_term_string_hook/2   with  the  same
+%   signature. This hook  may  use   alternative  serialization  such as
+%   fast_term_serialized/2,  use  library(ssl)  to    realise  encrypted
+%   messages, etc.
+
+udp_term_string(Term, String) :-
+    udp_term_string_hook(Term, String),
+    !.
+udp_term_string(Term, String) :-
+    (   var(String)
+    ->  format(string(String), '%-prolog-\n~W',
+               [ Term,
+                 [ ignore_ops(true),
+                   quoted(true)
+                 ]
+               ])
+    ;   sub_string(String, 0, _, _, '%-prolog-\n'),
+        term_string(Term, String,
+                    [ syntax_errors(quiet)
+                    ])
+    ).
+
